/*------------------------------------------------------------------------------
* rnx2rtkp.c : read rinex obs/nav files and compute receiver positions
*
*          Copyright (C) 2007 by T.TAKASU, All rights reserved.
*
* options : -DLIBRTKP = libary mode
*
* version : $Revision: 3 $ $Date: 07/05/09 14:13 $
* history : 2007/01/16  1.0 new
*           2007/03/15  1.1 add library mode
*           2007/05/08  1.2 separate from postpos.c
*-----------------------------------------------------------------------------*/
#include <stdarg.h>
#include "rtklib.h"

static const char rcsid[]="$Id: rnx2rtkp.c,v 1.11 2007/03/20 08:49:55 TTAKA Exp TTAKA $";

#define MAXFILE     4       /* max number of input files */

/* help text -----------------------------------------------------------------*/
static const char *help[]={
"",
" usage: rnx2rtkp [option]... file file [...]",
"",
" Read RINEX OBS/NAV files, compute receiver (rover) positions and output",
" position solutions. The first RINEX OBS file shall contain receiver (rover)",
" observations. For the relative mode, the second RINEX OBS file shall contain",
" reference (base) receiver observations. At least one RINEX NAV file shall be",
" included in input files. Command options are as follows. ([]:default)",
"",
" -h        print help",
" -o output output file [stdout]",
" -ts ds ts start day/time (ds=y/m/d ts=h:m:s) [obs start time]",
" -te de te end day/time   (de=y/m/d te=h:m:s) [obs end time]",
" -ti tint  time interval (sec) [all]",
" -p mode   mode (0:single,1:dgps,2:kinematic,3:static) [2]",
" -m mask   elevation mask angle (deg) [10]",
" -f freq   number of frequencies for relative mode (1:L1,2:L1+L2) [2]",
" -v thres  validation threshold for integer ambiguity (0.0:no AR) [3.0]",
" -b        backward solutions [off]",
" -c        forward/backward combined solutions [off]",
" -i        instantaneous integer ambiguity resolution [off]",
" -e        output x/y/z-ecef position [latitude/longitude/height]",
" -n        output NMEA-0183 GGA sentence [off]",
" -g        output latitude/longitude in the form of ddd mm ss.ss' [ddd.ddd]",
" -t        output time in the form of yyyy/mm/dd hh:mm:ss.ss [sssss.ss]",
" -u        output time in utc [gpst]",
" -d col    columns of time under decimal point [3]",
" -s sep    field separator [' ']",
" -r x y z  reference (base) receiver ecef pos (m) [average of single pos]",
" -cf file  correction file [no correction]"
};
/* print message -------------------------------------------------------------*/
extern void printmsg(const char *format, ...)
{
    va_list arg;
    va_start(arg,format); vfprintf(stderr,format,arg); va_end(arg);
    fprintf(stderr,"\n");
}
extern int printprog(int prog)
{
    return 0;
}
/* print help ----------------------------------------------------------------*/
static void printhelp(void)
{
    int i;
    for (i=0;i<sizeof(help)/sizeof(*help);i++) fprintf(stderr,"%s\n",help[i]);
    exit(0);
}
/* rnx2rtkp main -------------------------------------------------------------*/
int main(int argc, char **argv)
{
    gtime_t ts,te;
    int i,j,nfile,dirs=0,times=0,timef=0,timeu=3,posf=0,degf=0;
    double tint=0.0,es[]={1980,1,1,0,0,0},ee[]={2037,12,31,23,59,59};
    char *infile[MAXFILE],*outfile="",corrfile[256]="",*sep=" ";
    opt_t opt=opt_default;
    
    for (i=1,nfile=0;i<argc;i++) {
        if      (!strcmp(argv[i],"-o")&&i+1<argc) outfile=argv[++i];
        else if (!strcmp(argv[i],"-ts")&&i+2<argc) {
            sscanf(argv[++i],"%lf/%lf/%lf",es,es+1,es+2);
            sscanf(argv[++i],"%lf:%lf:%lf",es+3,es+4,es+5);
        }
        else if (!strcmp(argv[i],"-te")&&i+2<argc) {
            sscanf(argv[++i],"%lf/%lf/%lf",ee,ee+1,ee+2);
            sscanf(argv[++i],"%lf:%lf:%lf",ee+3,ee+4,ee+5);
        }
        else if (!strcmp(argv[i],"-ti")&&i+1<argc) tint=atof(argv[++i]);
        else if (!strcmp(argv[i],"-p")&&i+1<argc) opt.mode=atoi(argv[++i]);
        else if (!strcmp(argv[i],"-f")&&i+1<argc) opt.nf=atoi(argv[++i]);
        else if (!strcmp(argv[i],"-m")&&i+1<argc) opt.elmin=atof(argv[++i])*D2R;
        else if (!strcmp(argv[i],"-v")&&i+1<argc) opt.thresar=atof(argv[++i]);
        else if (!strcmp(argv[i],"-s")&&i+1<argc) sep=argv[++i];
        else if (!strcmp(argv[i],"-d")&&i+1<argc) timeu=atoi(argv[++i]);
        else if (!strcmp(argv[i],"-b")) dirs=1;
        else if (!strcmp(argv[i],"-c")) dirs=2;
        else if (!strcmp(argv[i],"-i")) opt.modear=1;
        else if (!strcmp(argv[i],"-u")) times=1;
        else if (!strcmp(argv[i],"-t")) timef=1;
        else if (!strcmp(argv[i],"-e")) posf=1;
        else if (!strcmp(argv[i],"-n")) posf=2;
        else if (!strcmp(argv[i],"-g")) degf=1;
        else if (!strcmp(argv[i],"-r")&&i+3<argc)
            for (j=0;j<3;j++) opt.rb[j]=atof(argv[++i]);
        else if (!strcmp(argv[i],"-cf")&&i+1<argc) strcpy(corrfile,argv[++i]);
        else if (*argv[i]=='-') printhelp();
        else if (nfile<MAXFILE) infile[nfile++]=argv[i];
    }
    if (opt.mode<0||3<opt.mode||opt.nf<1||2<opt.nf||opt.elmin<0.0||
        PI/2.0<opt.elmin||12<timeu) {
        printmsg("error : command option error");
        return -1;
    }
    ts=epoch2time(es);
    te=epoch2time(ee);
    if (nfile<=0) {
        printmsg("error : no input file");
        return -2;
    }
    return postpos(ts,te,tint,&opt,sep,timeu,dirs,times,timef,posf,degf,
                   infile,nfile,outfile,corrfile);
}
